@@ -11,4 +11,7 @@ class IAdminAuthzPolicy(Interface):
     """ Admin authorization policy"""
 
 class IColumnRenderer(Interface):
-    """ Table column renderer """
\ No newline at end of file
+    """ Table column renderer """
+
+class IQueryFilter(Interface):
+    """ Query filter """
\ No newline at end of file
