@@ -9,17 +9,21 @@ from sqlalchemy import orm, or_, types
 from pyramid.httpexceptions import HTTPFound, HTTPNotFound
 from pyramid.renderers import render_to_response
 from pyramid.response import Response, IResponse
-from pyramid.config.views import ViewDeriver
+from pyramid.decorator import reify
+from pyramid.i18n import TranslationStringFactory, get_localizer
 from jinja2 import Markup
 from webhelpers import util, paginate
 from webhelpers.html import HTML
 
 
 from pyramid_admin.forms import model_form
-from pyramid_admin.interfaces import IColumnRenderer, ISqlaSessionFactory
+from pyramid_admin.filters import LikeFilter, QuickBoolFilter, QueryFilter
+from pyramid_admin.interfaces import IColumnRenderer, ISqlaSessionFactory, IQueryFilter
 from pyramid_admin.utils import get_pk_column, get_pk_value
 
 
+_ = TranslationStringFactory('pyramid_admin')
+
 def action(name=None, **kw):
     """action decorator"""
     def _wrapper(fn):
@@ -30,6 +34,18 @@ def action(name=None, **kw):
     return _wrapper
 
 
+def bulk_action(title, name=None, **kw):
+    """bulk action decorator"""
+    def _wrapper(fn):
+        kw['name'] = name or fn.__name__
+        kw['fn_name'] = fn.__name__
+        kw['bulk'] = True
+        fn.title = title
+        fn.__action_params__ = kw
+        return fn
+    return _wrapper
+
+
 def column(label):
     """table column decorator"""
     def _wrap(fn):
@@ -47,12 +63,17 @@ class AdminViewMeta(type):
     def __new__(cls, name, bases, dct):
         ncl = type.__new__(cls, name, bases, dct)
         ncl.__actions__ = {}
+        ncl.bulk_actions = []
         for name in dir(ncl):
             val = getattr(ncl, name, None)
             if callable(val) and hasattr(val, '__action_params__'):
                 name = val.__action_params__["name"]
+                if name in ncl.not_allowed:
+                    continue
                 val.__action_params__.update(ncl.override.get(name, {}))
                 ncl.__actions__[name] = val
+                if val.__action_params__.get("bulk"):
+                    ncl.bulk_actions.append((name, val.title))
         return ncl
 
 
@@ -61,8 +82,8 @@ class AdminView(object):
 
     __metaclass__ = AdminViewMeta
 
-    field_list = ['id', 'repr']
-    list_links = ['id']
+    field_list = ['pk', 'repr']
+    list_links = ['pk']
     filters = []
     items_per_page = 20
     override = {}
@@ -83,14 +104,28 @@ class AdminView(object):
         self.request.view = self
         self.session = site.session
         self.parts = request.matchdict
+        self.localizer = get_localizer(request)
         self.list_order = {'field': self.request.GET.get('order'), 'desc': self.request.GET.get('desc')}
         for i, f in enumerate(self.filters):
-            f.id = 'f%s' % i
-            f.activate(self.request)
+            if isinstance(f, basestring) and hasattr(self.model, f):
+                typ = get_type(self.model, f)
+                filter_class = self.request.registry.queryAdapter(typ, IQueryFilter)
+                if not filter_class:
+                    continue
+                name, label = self.get_name_label(f)
+                filter_inst = filter_class(name, label + ':')
+                self.filters.pop(i)
+                self.filters.insert(i, filter_inst)
+            elif isinstance(f, QueryFilter):    
+                filter_inst = f
+            else:
+                raise TypeError("Invalid Filter type ''. \
+                    Must by a name of model field or Subclass of pyramid_admin.filters.QueryFilter" % type(f))
+            fid = 'f%s' % i
+            filter_inst.activate(self.request, fid)
 
     def __call__(self):
         action_name = self.parts.get("action", "list")
-
         if action_name in self.not_allowed:
             raise HTTPNotFound
         action = self.__actions__.get(action_name, None)
@@ -125,6 +160,11 @@ class AdminView(object):
             raise HTTPNotFound
         return obj
 
+    def get_bulk_selected(self):
+        ids = self.request.POST.getall('select')
+        objects = self.session.query(self.model).filter(get_pk_column(self.model).in_(ids)).all()
+        return objects
+
     def get_list_query(self):
         q = self.session.query(self.model)
         order = self.list_order['field']
@@ -173,11 +213,14 @@ class AdminView(object):
             form = self.get_form(formdata=self.request.POST)
             if form.validate():
                 form.populate_obj(obj)
-                self.pre_update(obj)
+                self.before_update(obj)
                 self.session.add(obj)
                 self.session.flush()
                 self.commit()
-                return HTTPFound(self.url())
+                msg = _('Object <strong>"${obj}"</strong> successfully updated.', mapping={'obj':self.repr(obj)})
+                self.message(msg)
+                next = 'new' if 'another' in self.request.POST else None
+                return HTTPFound(self.url(action=next))
         else:
             form = self.get_form(obj)
         return {'obj':obj, 'obj_form': form}
@@ -190,26 +233,27 @@ class AdminView(object):
             if form.validate():
                 obj = self.model()
                 form.populate_obj(obj)
-                self.pre_create(obj)
+                self.before_insert(obj)
                 self.session.add(obj)
                 self.session.flush()
                 self.commit()
-                return HTTPFound(self.url())
+                msg = _('New object <strong>"${obj}"</strong> successfully created.', mapping={'obj':self.repr(obj)})
+                self.message(msg)
+                next = 'new' if 'another' in self.request.POST else None
+                return HTTPFound(self.url(action=next))
         return {'obj':None, 'obj_form': form}
 
     @action(request_method="POST")
     def delete(self):
         obj = self.get_obj()
         self._delete_obj(obj)
-        self.commit()
         return HTTPFound(self.url())
 
-    @action(request_method="POST", renderer="pyramid_admin:templates/confirm_delete.jinja2")
+    @bulk_action(_("Delete selected"), request_method="POST", renderer="pyramid_admin:templates/confirm_delete.jinja2")
     def bulk_delete(self):
         if not self.is_allowed('delete'):
             raise HTTPNotFound
-        ids = self.request.POST.getall('select')
-        objects = self.session.query(self.model).filter(get_pk_column(self.model).in_(ids)).all()
+        objects = self.get_bulk_selected()
         if 'confirmed' not in self.request.POST:
             return {'bulk': True, "objects": objects}
         for obj in objects:
@@ -217,8 +261,10 @@ class AdminView(object):
         return HTTPFound(self.url())
 
     def _delete_obj(self, obj):
-        self.pre_delete(obj)
+        self.before_delete(obj)
         self.session.delete(obj)
+        msg = _('Object <strong>"${obj}"</strong> successfully deleted.', mapping={'obj':self.repr(obj)})
+        self.message(msg)
         self.commit()
 
     def commit(self):
@@ -230,15 +276,15 @@ class AdminView(object):
     def process_response(self, data):
         pass
 
-    def pre_create(self, obj):
+    def before_insert(self, obj):
         """pre object creation hook"""
         pass
 
-    def pre_update(self, obj):
+    def before_update(self, obj):
         """pre object update hook"""
         pass
 
-    def pre_delete(self, obj):
+    def before_delete(self, obj):
         """pre object deletion hook"""
         pass
 
@@ -252,28 +298,58 @@ class AdminView(object):
                           field_args=self.form_field_args, 
                           fields_override=self.form_fields)
 
-    def pk(self, obj):
-        return get_pk_value(obj)
+    def message(self, msg, type='success'):
+        self.request.session.flash(self.localizer.translate(msg), type)
 
+    def get_name_label(self, field_name):
+        """
+        """
+        form = self._blank_form
+        if ':' in field_name:
+            field_name, label = field_name.split(':', 1)
+        elif field_name in form:
+            label = form[field_name].label.text
+        else:
+            label = field_name.title().replace('_', ' ')
+        return field_name, label
+            
+    @reify
+    def _blank_form(self):
+        """
+        blank form instance
+        """
+        return self.get_form()        
+
+    @reify
     def columns(self):
+        colls = []
         for f in self.field_list:
-            if isinstance(f, basestring) and hasattr(self.model, f):
-                yield Column(self, f)
-            elif isinstance(f, basestring) and hasattr(self, f):
-                yield MethodColumn(self, f)
-            else:
-                raise AttributeError("Invalid column name '%s'" % f)
+            if isinstance(f, basestring):
+                name, label = self.get_name_label(f)
+                if hasattr(self.model, name):
+                    colls.append(Column(self, name, label))
+                elif hasattr(self, name):
+                    colls.append(MethodColumn(self, name))
+        return colls
+
+    @column("#")
+    def pk(self, obj):
+        """
+        value of object primary key
+        """
+        return get_pk_value(obj)
 
     @column("")
     def repr(self, obj):
         return unicode(obj)
 
+
     
 class Column(object):
 
-    def __init__(self, view, name):
+    def __init__(self, view, name, label=None):
         self.view = view
-        self.label = name
+        self.label = label or name
         self.name = name
 
     def title(self):
@@ -287,7 +363,7 @@ class Column(object):
         elif field_name == self.view.list_order['field'] and self.view.list_order['desc']:
             order_ico = '<i class="icon-chevron-up"/>'
             url = util.update_params(url, order=None, desc=None)
-        return Markup('<a href="%s">%s</a> %s' % (url, field_name, order_ico))
+        return Markup('<a href="%s">%s</a> %s' % (url, self.label, order_ico))
 
     def get_val(self, obj):
         renderer = self.view.request.registry.queryAdapter(get_type(obj, self.name), IColumnRenderer)
@@ -339,6 +415,11 @@ class BoolRenderer(object):
     def __call__(self, val, editable=False):
         return Markup('<i class="%s"></i>' % ('icon-ok' if val else 'icon-remove'))
 
+def like_filter_factory(typ):
+    return LikeFilter
+
+def bool_filter_factory(typ):
+    return QuickBoolFilter
 
 def register_adapters(reg):
     from sqlalchemy.types import Integer
@@ -350,6 +431,8 @@ def register_adapters(reg):
     reg.registerAdapter(StringRenderer, (Integer,), IColumnRenderer)
     reg.registerAdapter(StringRenderer, (String,), IColumnRenderer)
     reg.registerAdapter(BoolRenderer, (Boolean,), IColumnRenderer)
+    reg.registerAdapter(like_filter_factory, (String,), IQueryFilter)
+    reg.registerAdapter(bool_filter_factory, (Boolean,), IQueryFilter)
 
 
 def to_dict(obj):
